#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>

#include <stdio.h>
#include <stdlib.h>

#include "xmps.h"


typedef struct _Process {
     int pid;
     int ppid;
     String data;
     struct _Process *child;
     int childCount;
     struct _Process *brother;
     struct _Process *next;
     struct _Process *nexthash;
} *Process;

typedef struct _ProcessList {
     Process first;
     Process *hashtable;
} *ProcessList;

typedef struct _Field {
     int start;
     int end;
} *Field;


Field pidField;
Field ppidField;
Field commandField;

int indentation = 2;
int hashvalue = 97;


Field newField (x, y)
     int x;
     int y;
{
     Field result;

     result = (Field) XtMalloc (sizeof (*result));

     result->start = x;
     result->end = y;

     return (result);
}


void deleteField (field)
     Field field;
{
     if (field != NULL) 
	  XtFree (field);
}


Field fieldPosition (string, name)
     String string;
     String name;
{
     Field field = newField (0, 0);
     String origin = string;
     String endOfString = string + strlen (string);

     /*
      * Move to which'th field.
      */
     field->start = string - origin;
     string = string + strspn (string, " ");		/* Skip blank(s) */
     while (string < endOfString) {
	  if (!strncmp (string, name, strlen (name))) {
	       if ((string = strchr (string, ' ')) != NULL)
		    field->end = string - origin;	/* To next blank(s) */
	       else
		    field->end = endOfString - origin;
	       return (field);
	  }
	  if ((string = strchr (string, ' ')) == NULL)
	       return (NULL);				/* To next blank(s) */
	  field->start = string - origin;
	  string = string + strspn (string, " ");	/* Skip them */
     }

     return (NULL);
}


String fieldValue (string, field)
     String string;
     Field field;
{
     char value[32];
     int length = field->end - field->start;
     int i;

     string = string + field->start;
     strncpy (value, string, length);
     value[length] = '\0';

     return (value);
}


String pid (string)
     String string;
{
     return (fieldValue (string, pidField));
}


String ppid (string)
     String string;
{
     return (fieldValue (string, ppidField));
}


void parseHeader (string)
     String string;
{
     deleteField (pidField);
     deleteField (ppidField);
     deleteField (commandField);
     
     pidField = fieldPosition (string, "PID");
     ppidField = fieldPosition (string, "PPID");
     if (ppidField == NULL)
	  ppidField = pidField;
     commandField = fieldPosition (string, "COM");
}


Process newProcess (string)
     String string;
{
     Process result;

     result = (Process) XtMalloc (sizeof (*result));
     result->data = string;
     result->pid = atoi (pid (string));
     result->ppid = atoi (ppid (string));
     result->child = NULL;
     result->brother = NULL;
     result->next = NULL;
     result->nexthash = NULL;

     return (result);
}


void deleteProcess (process)
     Process process;
{
     if (process != NULL) 
	  XtFree (process);
}


int hash (id)
     int id;
{
     return (id % hashvalue);
}


ProcessList newProcessList ()
{
     ProcessList result;
     int i;

     result = (ProcessList) XtMalloc (sizeof (*result));
     result->first = NULL;
     result->hashtable = (Process) XtMalloc (sizeof (Process) *
					     (hashvalue + 1));
     for (i = 0; i < hashvalue; i++)
	  result->hashtable[i] = NULL;

     return (result);
}


void deleteProcessList (list)
     ProcessList list;
{
     Process current = list->first;
     Process next;

     while (current != NULL) {
	  next = current->next;
	  deleteProcess (current);
	  current = next;
     }

     XtFree (list->hashtable);
     XtFree (list);
}


void addProcess (list, process)
     ProcessList list;
     Process process;
{
     Process current;
     
     /*
      * Add process to list.
      */
     process->next = list->first;
     list->first = process;
}


void hashAddProcess (list, process)
     ProcessList list;
     Process process;
{
     Process current;
     
     /*
      * Add process to list.
      */
     addProcess (list, process);

     /*
      * Setup the hashtable for this process
      * for a faster access.
      */
     current = list->hashtable[hash (process->pid)];
     if (current == NULL)
	  list->hashtable[hash (process->pid)] = process;
     else {
	  Process previous;

	  while (current != NULL) {
	       previous = current;
	       current = current->nexthash;
	  }
	  process->nexthash = current;
	  previous->nexthash = process;
     }
}


void addChild (parent, child)
     Process parent;
     Process child;
{
     /*
      * Add child as a child of parent.
      * Sort childs by pid.
      */
     if (parent->child == NULL)
	  parent->child = child;
     else if (parent->child->pid > child->pid) {
	  child->brother = parent->child;
	  parent->child = child;
     } else {
	  Process current;
	  Process previous;

	  current = parent->child;
	  while ((current != NULL) && (child->pid >= current->pid)) {
	       previous = current;
	       current = current->brother;
	  }
	  
	  child->brother = current;
	  previous->brother = child;
     }
}


void sortAddChild (parent, child)
     Process parent;
     Process child;
{
     /*
      * Add child as a child of parent.
      * Sort childs by number of number of children.
      */
     if (parent->child == NULL)
	  parent->child = child;
     else if (parent->child->childCount > child->childCount) {
	  child->brother = parent->child;
	  parent->child = child;
     } else {
	  Process current;
	  Process previous;

	  current = parent->child;
	  while ((current != NULL) &&
		 (child->childCount >= current->childCount)) {
	       previous = current;
	       current = current->brother;
	  }
	  
	  child->brother = current;
	  previous->brother = child;
     }
}


Process parentProcess (list, process)
     ProcessList list;
     Process process;
{
     Process current;

     /*
      * Exit now if the process is its own parent.
      */
     if (process->pid == process->ppid)
	  return (NULL);
     
     /*
      * Look for the parent of process.
      * This is a rather expensive method of doing it, but it will do for now.
      */
     
#ifdef notdef
     current = list->first;
     while ((current != NULL) && (current->pid != process->ppid))
	  current = current->next;
#else
     current = list->hashtable[hash (process->ppid)];
     while ((current != NULL) && (current->pid != process->ppid))
	  current = current->nexthash;
#endif

     return (current);
}


int countChilds (parent)
     Process parent;
{
     Process child = parent->child;
     int childCount = 0;

     while (child != NULL) {
	  childCount = childCount + countChilds (child);
	  child = child->brother;
     }
     parent->childCount = childCount;

     return (childCount + 1);
}


void sort (parent)
     Process parent;
{
     Process child = parent->child;
     Process next;

     parent->child = NULL;
     while (child != NULL) {
	  next = child->brother;
	  child->brother = NULL;
	  sortAddChild (parent, child);
	  sort (child);
	  child = next;
     }
}


String indent (string, level)
     String string;
     int level;
{
     String result = (String) XtMalloc (strlen (string)
					+ indentation * level
					+ 2);
     int separation = commandField->start;

     /*
      * Some fields overrun, so we need to go to the real start of
      * the command (if any).
      */
     if (string[separation] && (string [separation] != ' ')) {
	  separation = strchr (string + separation, ' ') - string;
	  separation = separation + strspn (string + separation, " ");
     }

     /*
      * Use printf to indent the command within string.
      */
     sprintf (result,
	      "%.*s%*s%-s\n",
	      separation, string,
	      indentation * level, "",
	      string + separation);

     return (result);
}


String indentHierarchy (current, level)
     Process current;
     int level;
{
     String result = XtNewString ("");
     String partialResult;

     /*
      * Do me first.
      */
     partialResult = indent (current->data, level);
     result = XtRealloc (result,
			 strlen (result) + strlen (partialResult) + 1);
     strcat (result, partialResult);
     XtFree (partialResult);

     /*
      * Do all my childs.
      */
     if (current->child != NULL) {
	  partialResult = indentHierarchy (current->child, level + 1);
	  result = XtRealloc (result,
			      strlen (result) + strlen (partialResult) + 1);
	  strcat (result, partialResult);
	  XtFree (partialResult);
     }

     /*
      * Finally, do all my brothers.
      */
     if (current->brother != NULL) {
	  partialResult = indentHierarchy (current->brother, level);
	  result = XtRealloc (result,
			      strlen (result) + strlen (partialResult) + 1);
	  strcat (result, partialResult);
	  XtFree (partialResult);
     }

     return (result);
}


String format (lines)
     String lines;
{
     String line;
     String result;
     Process process;
     ProcessList processList = newProcessList ();
     ProcessList orphanList = newProcessList ();

     if ((line = strtok (lines, "\n")) == NULL)
	  return (lines);

     parseHeader (line);
     
     /*
      * Store the header first.
      */
     result = XtNewString (line);
     result = XtRealloc (result, strlen (result) + 1);
     strcat (result, "\n");

     /*
      * Transform the input string into a linked list of processes.
      */
     while ((line = strtok (NULL, "\n")) != NULL) {
	  process = newProcess (line);
	  hashAddProcess (processList, process);
     }

     /*
      * For each process, find its parent.
      * If it does not exist, then that process is a root process.
      */
     process = processList->first;
     while (process != NULL) {
	  /* Save next since addProcess overrides it */
	  Process next = process->next;
	  Process parent;

	  parent = parentProcess (processList, process);
	  if (parent != NULL)
	       addChild (parent, process);
	  else
	       addProcess (orphanList, process);
	  process = next;
     }

     /*
      * Compute the number of children per child, so that we can print
      * those processes that have fewer chidren first.
      */
     process = orphanList->first;
     while (process != NULL) {
	  (void) countChilds (process);
	  process = process->next;
     }
     
     /*
      * Sort the process list based on the number of children
      * per process.
      */
     process = orphanList->first;
     while (process != NULL) {
	  sort (process);
	  process = process->next;
     }
     
     /*
      * Sort the process list according to a parent-child relationship.
      */
     process = orphanList->first;
     while (process != NULL) {
	  String partialResult;

	  partialResult = indentHierarchy (process, 0);
	  result = XtRealloc (result,
			      strlen (result) + strlen (partialResult) + 1);
	  strcat (result, partialResult);
	  XtFree (partialResult);
	  
	  process = process->next;
     }

     /*
      * Free all the garbage we have generated.
      */
     deleteProcessList (processList);
     deleteProcessList (orphanList);

     return (result);
}
