#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "xmps.h"


void verifyOptions ()
{
     struct stat ps;

     if (stat (resources.psCommand, &ps) == -1) {
	  error ("No such ps command :\n%s",
		 resources.psCommand, (char *) 0);
	  exit (-1);
     }

     if (resources.beautify) {
	  if (strlen (resources.beautifyCommand)) {
	       struct stat pspp;

	       if (stat (resources.beautifyCommand, &pspp) == -1) {
		    error ("No such beautifier :\n%s",
			   resources.beautifyCommand, (char *) 0);
		    resources.beautify = False;
	       }
	  } else
	       internalBeautifier = True;
     }
     
     if (resources.fullListing && resources.longListing)
	  resources.fullListing = False;

     if (resources.noNonTerminal && ! resources.noGroupLeader)
	  resources.noGroupLeader = True;
     
     if (resources.allProcesses && resources.noNonTerminal) {
	  resources.noNonTerminal = False;
	  resources.noGroupLeader = False;
     }

     if (resources.allProcesses && resources.noGroupLeader)
	  resources.noGroupLeader = False;

     if (resources.periodicRefresh)
	  setTimer (True);
}


int changeOption (widget, menuSelected, buttonInfo)
     Widget widget;
     int menuSelected;
     XmToggleButtonCallbackStruct *buttonInfo;
{
     switch (menuSelected) {
	case MENU_AUTOMATIC_REFRESH:
	  resources.automaticRefresh = buttonInfo->set;
	  break;
	  
	case MENU_PERIODIC_REFRESH:
	  resources.periodicRefresh = buttonInfo->set;
	  setTimer (buttonInfo->set);
	  break;
	  
	case MENU_AUTOMATIC_MEMORY:
	  resources.automaticMemory = buttonInfo->set;
	  break;
	  
	case MENU_BEAUTIFY:
	  if (buttonInfo->set == True) {
	       if (!internalBeautifier) {
		    struct stat pspp;

		    if (stat (resources.beautifyCommand, &pspp) == -1) {
			 error ("No beautify command :\n%s",
				resources.beautifyCommand, (char *) 0);
		    } else
			 resources.beautify = True;
	       } else
		    resources.beautify = True;
	  } else
	       resources.beautify = False;

	  reDisplayProcessesIfNecessary ();
	  break;

	case MENU_MULTIPLE_OCCURRENCES:
	  resources.multipleOccurrences = buttonInfo->set;
	  break;
	  
	case MENU_IGNORE_CASE:
	  resources.ignoreCase = buttonInfo->set;
	  break;

	case MENU_PRINT_SELECTED:
	  resources.printSelected = buttonInfo->set;
	  break;
	  
	case MENU_FULL_LISTING:
	  if (buttonInfo->set == True) {
	       resources.fullListing = True;
	       resources.longListing = False;
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "longListing"),
				       False, False);
	  } else
	       resources.fullListing = False;

	  reDisplayProcessesIfNecessary ();
	  break;

	case MENU_LONG_LISTING:
	  if (buttonInfo->set == True) {
	       resources.fullListing = False;
	       resources.longListing = True;
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "fullListing"),
				       False, False);
	  } else
	       resources.longListing = False;

	  reDisplayProcessesIfNecessary ();
	  break;

	case MENU_CLUSTER_WIDE:
	  resources.clusterWide = buttonInfo->set;
	  reDisplayProcessesIfNecessary ();
	  break;
	  

	case MENU_ALL_PROCESSES:
	  if (buttonInfo->set == True) {
	       resources.allProcesses = True;
	       resources.noGroupLeader = False;
	       resources.noNonTerminal = False;
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "noGroupLeader"),
				       False, False);
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "noNonTerminal"),
				       False, False);
	       disableUser ();
	       disableGroup ();
	       disableTerminal ();
	  } else {
	       resources.allProcesses = False;
	       enableUser ();
	       enableGroup ();
	       enableTerminal ();
	  }

	  reDisplayProcessesIfNecessary ();
	  break;

	case MENU_NO_GROUP_LEADER:
	  if (buttonInfo->set == True) {
	       resources.allProcesses = False;
	       resources.noGroupLeader = True;
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "allProcesses"),
				       False, False);
	       disableUser ();
	       disableGroup ();
	       disableTerminal ();
	  } else {
	       resources.noGroupLeader = False;
	       if (resources.noNonTerminal == True) {
		    resources.noNonTerminal = False;
		    XmToggleButtonSetState (XtNameToWidget (viewPane,
							    "noNonTerminal"),
					    False, False);
	       }
	       enableUser ();
	       enableGroup ();
	       enableTerminal ();
	  }

	  reDisplayProcessesIfNecessary ();
	  break;

	case MENU_NO_NON_TERMINAL:
	  if (buttonInfo->set == True) {
	       resources.allProcesses = False;
	       resources.noGroupLeader = True;
	       resources.noNonTerminal = True;
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "allProcesses"),
				       False, False);
	       XmToggleButtonSetState (XtNameToWidget (viewPane,
						       "noGroupLeader"),
				       True, False);
	       disableUser ();
	       disableGroup ();
	       disableTerminal ();
	  } else {
	       resources.noNonTerminal = False;
	       if (! resources.noGroupLeader) {
		    enableUser ();
		    enableGroup ();
		    enableTerminal ();
	       }
	  }

	  reDisplayProcessesIfNecessary ();
	  break;


	default:
	  error ("Invalid menu selection :\ninternal error", (char *) 0);
	  break;
     }
}


