#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <Xm/List.h>
#include <Xm/Text.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <sys/utsname.h>

#include "xmps.h"


int selectAllProcesses (widget, clientData, callData)
     Widget widget;
     caddr_t clientData;
     caddr_t callData;
{
     int itemCount;
     int current;

     Arg arglist[2];
     Cardinal i;
     
     i = 0;
     XtSetArg (arglist[i], XmNitemCount, &itemCount); i++;
     XtGetValues (listArea, arglist, i);
     
     XtUnmapWidget (processArea);
     XmListDeselectAllItems (listArea);
     XmListSetAddMode (listArea, True);
     for (current = 1; current <= itemCount; current++)
	  XmListSelectPos (listArea, current, False);
     XmListSetAddMode (listArea, False);
     XtMapWidget (processArea);
}


int unselectAllProcesses (widget, clientData, callData)
     Widget widget;
     caddr_t clientData;
     caddr_t callData;
{
     XtUnmapWidget (processArea);
     XmListDeselectAllItems (listArea);
     XtMapWidget (processArea);
}


void highlightProcess (processPosition)
     int processPosition;
{
     XmListSelectPos (listArea, processPosition, True);
}


void centerProcess (processPosition, topProcessPosition, visibleProcessCount)
     int processPosition;
     int topProcessPosition;
     int visibleProcessCount;
{
     /*
      * Center the requested process.
      */
     if (! ((processPosition >= topProcessPosition) &&
	    (processPosition <= (topProcessPosition +
				 visibleProcessCount - 1)))) {
	  if (processPosition > visibleProcessCount/2) {
	       XmListSetPos (listArea, processPosition - visibleProcessCount/2);
	  } else {
	       XmListSetPos (listArea, processPosition);
	  }
     }
}


void reDisplayProcessesIfNecessary ()
{
     if (resources.automaticRefresh)
	  reDisplayProcesses ();
}


int reDisplayProcesses (widget, clientData, callData)
     Widget widget;
     caddr_t clientData;
     caddr_t callData;
{
     int oldTopItemPosition;
     int oldItemCount;
     int itemCount;
     int topItemPosition;

     Arg arglist[4];
     Cardinal i;
     
     extern void displayProcesses ();

     /*
      * Disable alarms while we refresh the display.
      */
     if (resources.periodicRefresh)
	  setTimer (False);
     
     /*
      * Hide the list while we update it.
      */
     setBusyIndicator ();
     XtUnmapWidget (processArea);

     /*
      * Remember where the top of the list is.
      */
     i = 0;
     XtSetArg (arglist[i], XmNtopItemPosition, &oldTopItemPosition); i++;
     XtSetArg (arglist[i], XmNitemCount, &oldItemCount); i++;
     XtGetValues (listArea, arglist, i);
     
     /*
      * Xm bug fix.
      * Go to the beginning of the list.
      */
     XmListSetPos (listArea, 1);
     XmListSetHorizPos (listArea, 1);

     /*
      * Recreate the processes information.
      */
     XmListDeleteAllItems (listArea);
     displayProcesses ();

     /*
      * Go back we were.
      */
     i = 0;
     XtSetArg (arglist[i], XmNitemCount, &itemCount); i++;
     XtGetValues (listArea, arglist, i);
     
     i = 0;
     if (itemCount >= oldItemCount)
	  topItemPosition = oldTopItemPosition;
     else {
	  topItemPosition = oldTopItemPosition - (oldItemCount - itemCount);
	  if (topItemPosition <= 0)
	       topItemPosition = 1;
     }
     XtSetArg (arglist[i], XmNtopItemPosition, topItemPosition); i++;
     XtSetValues (listArea, arglist, i);

     /*
      * Make the list visible again.
      */
     XtMapWidget (processArea);
     unsetBusyIndicator ();

     /*
      * Reenable alarms.
      */
     if (resources.periodicRefresh)
	  setTimer (True);
}


int applyAction ()
{
     XmStringTable selectedItems;
     int selectedItemCount;
     int current;
     String line;
     char command[512];
     Boolean remote;
     struct utsname name;

     Arg arglist[4];
     Cardinal i;
     
     /*
      * Find out whether or not we need to use remsh.
      */
     if ((strcmp (currentHost, "localhost") == 0) ||
	 ((uname (&name) != -1) && (strcmp (currentHost, name.nodename) == 0)))
	  remote = False;
     else
	  remote = True;

     /*
      * Get the selected processes.
      */
     i = 0;
     XtSetArg (arglist[i], XmNselectedItems, &selectedItems); i++;
     XtSetArg (arglist[i], XmNselectedItemCount, &selectedItemCount); i++;
     XtGetValues (listArea, arglist, i);
     
     /*
      * Apply the current action on them.
      */
     setBusyIndicator ();
     for (current = 1; current <= selectedItemCount; current++) {
	  XmStringGetLtoR (selectedItems[current-1],
			   XmSTRING_DEFAULT_CHARSET,
			   &line);
	  if (remote)
	       sprintf (command, "%s %s -n %s %s",
			resources.rshCommand, currentHost,
			currentAction, pid (line));
	  else
	       sprintf (command, "%s %s", currentAction, pid (line));
	  system (command);
          XtFree (line);
     }
     unsetBusyIndicator ();
}


String buildCommandLine ()
{
     String command;
     struct utsname name;

     command = (char *) XtMalloc (sizeof (char) * 512);

     /*
      * Store the process command first.
      */
     if ((strcmp (currentHost, "localhost") == 0) ||
	 ((uname (&name) != -1) && (strcmp (currentHost, name.nodename) == 0)))
	  strcpy (command, resources.psCommand);
     else
	  sprintf (command,"%s %s -n %s",
		   resources.rshCommand, currentHost, resources.psCommand);
     
     /*
      * Add a hyphen now for eventual options and do not bother any more.
      */
     strcat (command, " -");
     
     /*
      * Add the process type options.
      */
     if (resources.allProcesses)
	  strcat (command, resources.allProcessesOption);
     else if (resources.noNonTerminal)
	  strcat (command, resources.noNonTerminalOption);
     else if (resources.noGroupLeader)
	  strcat (command, resources.noGroupLeaderOption);

     /*
      * Add the listing options.
      */
     if (resources.fullListing)
	  strcat (command, resources.fullListingOption);
     else if (resources.longListing)
	  strcat (command, resources.longListingOption);

     /*
      * Add the user name to look for.
      */
     if (strcmp (currentUser, "all") != 0) {
#ifdef notdef
	  struct passwd *passwd = getpwnam (currentUser);
	  if (passwd != NULL) {
	       sprintf (command, "%s -%s %d",
			command, resources.usersOption, passwd->pw_uid);
	  } else
	       error ("Cannot find selected user :\n%s",
		      currentUser, (char *) 0);
#else
	  sprintf (command, "%s -%s %s",
		   command, resources.usersOption, currentUser);
#endif
     }

     /*
      * Add the group name to look for.
      */
     if (strcmp (currentGroup, "all") != 0) {
#ifdef notdef
	  struct group *group = getgrnam (currentGroup);
	  if (group != NULL) {
	       sprintf (command, "%s -%s %d",
			command, resources.groupsOption, group->gr_gid);
	  } else
	       error ("Cannot find selected group :\n%s",
		      currentGroup, (char *) 0);
#else
	  sprintf (command, "%s -%s %s",
		   command, resources.groupsOption, currentGroup);
#endif
     }

     /*
      * Add the terminal to look for.
      */
     if (strcmp (currentTerminal, "") != 0)
	  sprintf (command, "%s -%s %s",
		   command, resources.terminalsOption, currentTerminal);

     /*
      * Add the beautifier.
      */
     if (resources.beautify && !internalBeautifier)
	  sprintf (command, "%s | %s",
		   command, resources.beautifyCommand);

     return (command);
}


void displayProcesses ()
{
     char *argv[4];
     String command;
     String line;
     String lines;
     String oldLines;
     XmString label;
     int processCount = 0;

     Arg arglist[4];
     Cardinal i;

     extern void infoLineUpdate ();
     
     /*
      * Construct the appropriate command line.
      */
     command = buildCommandLine ();
     
     /*
      * Get processes information.
      */
     argv[0] = "/bin/sh";
     argv[1] = "-c";
     argv[2] = command;
     argv[3] = 0;
     lines = executeCommand (argv);

     /*
      * Format it eventually.
      */
     if (resources.beautify && internalBeautifier)
	  lines = format (lines);
     oldLines = lines;

     /*
      * Print the title line.
      */
     if ((line = strtok (lines, "\n")) != NULL) {
	  parseHeader (line);

	  label = XmStringCreateLtoR (line, XmSTRING_DEFAULT_CHARSET);
	  
	  i = 0;
	  XtSetArg (arglist[i], XmNlabelString, label); i++;
	  XtSetValues (headerArea, arglist, i);
	  XmStringFree (label);

	  /*
	   * Print each process information.
	   */
	  while ((line = strtok (NULL, "\n")) != NULL) {
	       label = XmStringCreateLtoR (line, XmSTRING_DEFAULT_CHARSET);
	       XmListAddItemUnselected (listArea, label, 0);
	       XmStringFree (label);
	       processCount = processCount + 1;
	  }
     }

     XtFree (oldLines);

     infoLineUpdate (processCount);
}


