#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <Xm/List.h>

#include <stdio.h>
#include <regex.h>

#include "xmps.h"


void setSearch (state)
     Boolean state;
{
     Arg arglist[2];
     Cardinal i;
     
     i = 0;
     XtSetArg (arglist[i], XmNsensitive, state); i++;
     XtSetValues (XtNameToWidget (processPane, "previous"), arglist, i);
     XtSetValues (XtNameToWidget (processPane, "next"), arglist, i);
}


void enableSearch ()
{
     setSearch (True);
}


void disableSearch ()
{
     setSearch (False);
}


int searchProcessPrompt (widget, clientData, callData)
     Widget widget;
     caddr_t clientData;
     caddr_t callData;
{
     extern int searchProcess ();
     
     if (! searchDialog) {
	  XmString title, label;
	  
	  Arg arglist[4];
	  Cardinal i;
	  
	  title = XmStringCreateLtoR ("Search Dialog",
				      XmSTRING_DEFAULT_CHARSET);
	  label = XmStringCreateLtoR ("Search For...",
				      XmSTRING_DEFAULT_CHARSET);

	  i = 0;
	  XtSetArg (arglist[i], XmNdialogTitle, title); i++;
	  XtSetArg (arglist[i], XmNselectionLabelString, label); i++;
	  searchDialog = (Widget) XmCreatePromptDialog (topLevel,
							"searchDialog",
							arglist,i);

	  XtUnmanageChild ((Widget) XmMessageBoxGetChild (searchDialog,
							  XmDIALOG_HELP_BUTTON));
	  XtAddCallback (searchDialog,
			 XmNokCallback,
			 (XtCallbackProc) searchProcess,
			 (XtPointer) ACTION_SEARCH);

	  XmStringFree (title);
	  XmStringFree (label);
     }
     
     XtManageChild (searchDialog);
}


int searchProcess (widget, typeOfSearch, callData)
     Widget widget;
     int typeOfSearch;
     caddr_t callData;
{
     XmStringTable items;
     XmString currentItem;
     String pattern;
     regex_t compiledPattern;
     int flag;
     String line;
     Boolean foundItem = False;
     int itemCount;
     int topItemPosition;
     int visibleItemCount;
     int current;
     enum {forward, backward} direction;
     static int lastSearchPosition = 1;
     int endOfSearchPosition;
     Boolean firstOccurrence = True;

     Arg arglist[5];
     Cardinal i;
     
     i = 0;
     XtSetArg (arglist[i], XmNitemCount, &itemCount); i++;
     XtSetArg (arglist[i], XmNtopItemPosition, &topItemPosition); i++;
     XtSetArg (arglist[i], XmNvisibleItemCount, &visibleItemCount); i++;
     XtSetArg (arglist[i], XmNitems, &items); i++;
     XtGetValues (listArea, arglist, i);
     
     /*
      * Disable temporarily the Next and Previous buttons.
      */
     disableSearch ();

     /*
      * Get the process name to search for.
      */
     i = 0;
     XtSetArg (arglist[i], XmNtextString, &currentItem); i++;
     XtGetValues (searchDialog, arglist, i);
     XmStringGetLtoR (currentItem, XmSTRING_DEFAULT_CHARSET, &pattern);

     if (strcmp (pattern, "") == 0) {
	  XtFree (pattern);
	  return;
     }

     flag = REG_EXTENDED | REG_NEWLINE | REG_NOSUB;
     if (resources.ignoreCase)
	  flag |= REG_ICASE;
     if (regcomp (&compiledPattern, pattern, flag) != 0) {
	  regerror ("not implemented");
	  XtFree (pattern);
	  return;
     }

     /*
      * Find where to search.
      */
     if (typeOfSearch == ACTION_SEARCH) {
	  current = lastSearchPosition;
	  endOfSearchPosition = current;
	  direction = forward;
     } else if (typeOfSearch == ACTION_SEARCH_NEXT) {
	  if (lastSearchPosition != itemCount)
	       current = lastSearchPosition + 1;
	  else
	       current = 1;
	  endOfSearchPosition = current;
	  direction = forward;
     } else if (typeOfSearch == ACTION_SEARCH_PREVIOUS) {
	  if (lastSearchPosition != 1)
	       current = lastSearchPosition - 1;
	  else
	       current = itemCount;
	  endOfSearchPosition = current;
	  direction = backward;
     } else {
	  error ("Invalid type of search :\ninternal error", (char *) 0);
          regfree (&compiledPattern);
          XtFree (pattern);
	  return;
     }
     
     /*
      * Search now.
      */
     XmListDeselectAllItems (listArea);
     XmListSetAddMode (listArea, True);
     do {
	  XmStringGetLtoR (items[current-1],
			   XmSTRING_DEFAULT_CHARSET,
			   &line);
	  if (regexec (&compiledPattern, line, (size_t) 0, NULL, 0) == 0) {
	       if (firstOccurrence)
		    centerProcess (current, topItemPosition, visibleItemCount);
	       highlightProcess (current);
	       foundItem = True;
	       if (! resources.multipleOccurrences)
		    break;
	       firstOccurrence = False;
	  }
	  if (((direction == forward) && (current < itemCount)) ||
	      ((direction == backward) && (current > 1)))
	       current += (direction == forward) ? 1 : -1;
	  else
	       current = (direction == forward) ? 1 : itemCount;
	  XtFree (line);
     } while (current != endOfSearchPosition);
     XmListSetAddMode (listArea, False);
     regfree (&compiledPattern);

     /*
      * Remember the position of the found item.
      */
     if (foundItem) {
	  lastSearchPosition = current;

	  if (! resources.multipleOccurrences)
	       enableSearch ();
     } else {
	  XmListDeselectAllItems (listArea);
	  error ("No such string :\n%s", pattern, (char *) 0);
     }
     XtFree (pattern);
}
