#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <Xm/Text.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xmps.h"


int menuParameterChange (widget, newValue, callData)
     Widget widget;
     String newValue;
     caddr_t callData;
{
     String widgetName = XtName (XtParent (widget));
     
     if (strcmp (widgetName, "hostPane") == 0) {
	  strcpy (currentHost, newValue);
	  reDisplayProcessesIfNecessary ();

     } else if (strcmp (widgetName, "userPane") == 0) {
	  strcpy (currentUser, newValue);
	  reDisplayProcessesIfNecessary ();

     } else if (strcmp (widgetName, "groupPane") == 0) {
	  strcpy (currentGroup, newValue);
	  reDisplayProcessesIfNecessary ();

     } else if (strcmp (widgetName, "terminalPane") == 0) {
	  strcpy (currentTerminal, newValue);
	  reDisplayProcessesIfNecessary ();

     } else if (strcmp (widgetName, "actionPane") == 0) {
	  strcpy (currentAction, newValue);
	  if (resources.automaticRefresh)
	       applyAction ();

     } else
	  error ("Invalid option menu selection :\ninternal error", (char *) 0);

     noInputMode ();
}


void textParameterUpdate (currentValue, newValue, menuPane)
     String currentValue;
     String newValue;
     Widget menuPane;
{
     if (! strcmp (currentValue, newValue))
	  return;
     
     strcpy (currentValue, newValue);
     
     if (resources.automaticMemory) {
	  Widget menuElement;
	  XmString label;
	  
	  Arg arglist[3];
	  Cardinal i;
	  
	  label = XmStringCreateLtoR (newValue, XmSTRING_DEFAULT_CHARSET);

	  i = 0;
	  XtSetArg (arglist[i], XmNlabelString, label); i++;
	  menuElement = (Widget) XmCreatePushButtonGadget (menuPane,
							   newValue,
							   arglist, i);
	  XtManageChild (menuElement);
	  XtAddCallback (menuElement,
			 XmNactivateCallback,
			 (XtCallbackProc) menuParameterChange, newValue);
	  
	  XmStringFree (label);
     }
}


void textParameterChange (widget, event, parameters, parameterCount)
     Widget widget;
     XEvent *event;
     String *parameters;
     Cardinal *parameterCount;
{
     String newValue = XmTextGetString (widget);
     Boolean error = False;
     Boolean noTerminal = False;

     String newUser;
     String newGroup;
     String newTerminal = "";
     String newHost;
     String newAction;

     extern void statusLineUpdate ();

     if (strstr (newValue, "()") != NULL)
	  noTerminal = True;
     
     if ((newUser = strtok (newValue, ":")) == NULL)
	  error = True;

     if ((newGroup = strtok (NULL, "(")) == NULL)
	  error = True;

     if (! noTerminal)
	  newTerminal = strtok (NULL, ")");

     if ((newHost = strtok (NULL, " ")) == NULL)
	  error = True;
     else
	  newHost = newHost + (noTerminal ? 2 : 1);

     if ((newAction = strtok (NULL, ")")) == NULL)
	  error = True;
     else
	  newAction = newAction + 2;

     if (!error) {
	  textParameterUpdate (currentUser, newUser, userPane);
	  textParameterUpdate (currentGroup, newGroup, groupPane);
	  textParameterUpdate (currentTerminal, newTerminal, terminalPane);
	  textParameterUpdate (currentHost, newHost, hostPane);
	  textParameterUpdate (currentAction, newAction, actionPane);

	  reDisplayProcessesIfNecessary ();
     }
     noInputMode ();

     XtFree (newValue);
}


void inputMode ()
{
     Widget statusArea = XtNameToWidget (topLevel,
					 "topWindow.selectionArea.statusArea");
     Widget textArea = XtNameToWidget (topLevel,
				       "topWindow.selectionArea.textArea");

     extern void textLineUpdate ();

     XtUnmanageChild (statusArea);
     textLineUpdate ();
     XtManageChild (textArea);
}


void noInputMode ()
{
     Widget statusArea = XtNameToWidget (topLevel,
					 "topWindow.selectionArea.statusArea");
     Widget textArea = XtNameToWidget (topLevel,
				       "topWindow.selectionArea.textArea");

     extern void statusLineUpdate ();

     XtUnmanageChild (textArea);
     statusLineUpdate ();
     XtManageChild (statusArea);
}


void infoLineUpdate (processCount)
     int processCount;
{
     Widget infoArea = XtNameToWidget (topLevel,
				       "topWindow.selectionArea.infoArea");
     XmString label;
     char infoLine[512];

     Arg arglist[2];
     Cardinal i;
     
     sprintf (infoLine, "%d Processes", processCount);

     label = XmStringCreateLtoR (infoLine, XmSTRING_DEFAULT_CHARSET);

     i = 0;
     XtSetArg (arglist[i], XmNlabelString, label); i++;
     XtSetValues (infoArea, arglist, i);

     XmStringFree (label);
}


void statusLineUpdate ()
{
     Widget statusArea = XtNameToWidget (topLevel,
					 "topWindow.selectionArea.statusArea");
     XmString label;
     char statusLine[512];

     Arg arglist[2];
     Cardinal i;
     
     sprintf (statusLine,
	      "%s:%s(%s)@%s  (%s)",
	      currentUser, currentGroup, currentTerminal, currentHost,
	      currentAction);

     label = XmStringCreateLtoR (statusLine, XmSTRING_DEFAULT_CHARSET);

     i = 0;
     XtSetArg (arglist[i], XmNlabelString, label); i++;
     XtSetValues (statusArea, arglist, i);

     XmStringFree (label);
}


void textLineUpdate ()
{
     Widget textArea = XtNameToWidget (topLevel,
				       "topWindow.selectionArea.textArea");
     char statusLine[512];
     
     sprintf (statusLine,
	      "%s:%s(%s)@%s  (%s)",
	      currentUser, currentGroup, currentTerminal, currentHost,
	      currentAction);

     XmTextSetString (textArea, statusLine);
}
