/*
 * Copyright (C) 1991, 1992 Jean-Jacques Moreau.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  The author makes no representations
 * about the suitability of this software for any purpose.  It is
 * provided "as is" without express or implied warranty.
 *
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef lint
static char copyright[] = "@(#) Copyright (c) 1991, 1992 Jean-Jacques Moreau\n\
 All rights reserved";
#endif

#ifndef lint
static char rcsid[] = "@(#)$Header: xmps.c,v 1.04 92/12/01 17:15:54 jjm Exp $";
#endif

static char *version = "Motif Process Browser Version 1.04\n\
(c) Copyright 1991,1992 Jean-Jacques Moreau, All Rights Reserved";


#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/DialogS.h>
#include <Xm/Frame.h>
#include <Xm/MainW.h>
#include <Xm/List.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/SeparatoG.h>
#include <Xm/Text.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#define GLOBAL
#include "xmps.h"
#include "options.h"
#include "menus.h"


String generalTranslations =
     "<Map>: ReDisplay()";

String textTranslations =
     "<Key>Return: UpdateParameter()\n\
     <Key>Escape: CancelInput()";

String statusTranslations =
     "<Btn1Down>: DoInput()\n\
     <Key>Return: DoInput()";

XtActionsRec xmPsActions[] = {
     { "ReDisplay", (XtActionProc) reDisplayProcessesIfNecessary},
     { "UpdateParameter", (XtActionProc) textParameterChange},
     { "DoInput", (XtActionProc) inputMode},
     { "CancelInput", (XtActionProc) noInputMode},
};


int versionXmPs (widget, clientData, callData)
     Widget widget;
     caddr_t clientData;
     caddr_t callData;
{
     Widget versionDialog = NULL;
     
     if (! versionDialog) {
	  XmString title, message;

	  Arg arglist[5];
	  Cardinal i;
	  
	  title = XmStringCreateLtoR ("Version Dialog",
				      XmSTRING_DEFAULT_CHARSET);
	  message = XmStringCreateLtoR (version,
					XmSTRING_DEFAULT_CHARSET);
	       
	  i = 0;
	  XtSetArg (arglist[i], XmNdialogTitle, title); i++;
	  XtSetArg (arglist[i], XmNmessageString, message); i++;
	  XtSetArg (arglist[i], XmNmessageAlignment, XmALIGNMENT_CENTER); i++;
	  XtSetArg (arglist[i], XmNdialogStyle, XmDIALOG_MODELESS); i++;
	  versionDialog = (Widget) XmCreateInformationDialog (topLevel,
							      "versionDialog",
							      arglist, i);

	  XtUnmanageChild ((Widget) XmMessageBoxGetChild (versionDialog,
							  XmDIALOG_CANCEL_BUTTON));
	  XtUnmanageChild ((Widget) XmMessageBoxGetChild (versionDialog,
							  XmDIALOG_HELP_BUTTON));

	  XmStringFree (title);
	  XmStringFree (message);
    }

    XtManageChild (versionDialog);
}


Widget createMenu (parent, menu, menuSize)
     Widget parent;
     MenuDescStruct menu[];
     int menuSize;
{
     Widget pane;
     Widget button;
     Widget menuElement;

     XmString label;
     XmString acceleratorText;
     Boolean sensitive, toggle;
     int j;
     
     Arg arglist[6];
     Cardinal i;
     
     /*
      * Create a pulldown menu.
      */
     pane = (Widget) XmCreatePulldownMenu (parent, concat (menu[0].id, "Pane"),
					   NULL, 0);

     /*
      * Create all the menu elements.
      */
     for (j = 1; j < menuSize; j++) {
	  int labelPosition = 0;

	  if (strcmp (menu[j].label, "-") == 0) {
	       XtManageChild (XmCreateSeparatorGadget (pane, "separator",
						       NULL, 0));
	  } else {
	       sensitive = True;
	       if (menu[j].label[labelPosition] == '-') {
		    sensitive = False;
		    labelPosition++;
	       }

	       toggle = False;
	       if (menu[j].label[labelPosition] == '^') {
		    toggle = True;
		    labelPosition++;
	       }
	       
	       label = XmStringCreateLtoR (&menu[j].label[labelPosition],
					   XmSTRING_DEFAULT_CHARSET);

	       acceleratorText = XmStringCreateLtoR (menu[j].acceleratorText,
						     XmSTRING_DEFAULT_CHARSET);

	       i = 0;
	       XtSetArg (arglist[i], XmNlabelString, label); i++;
	       if (menu[j].mnemonic != '-') {
		    XtSetArg (arglist[i],
			      XmNmnemonic, menu[j].mnemonic); i++;
		    XtSetArg (arglist[i],
			      XmNacceleratorText, acceleratorText); i++;
		    XtSetArg (arglist[i],
			      XmNaccelerator, menu[j].accelerator); i++;
	       }
	       XtSetArg (arglist[i], XmNsensitive, sensitive); i++;

	       if (! toggle)
		    menuElement = XmCreatePushButtonGadget (pane, menu[j].id,
							    arglist, i);
	       else {
		    menuElement = XmCreateToggleButtonGadget (pane, menu[j].id,
							      arglist, i);
		    XmToggleButtonSetState (menuElement,
					    *(menu[j].state), False);
	       }

	       XtManageChild (menuElement);

	       XtAddCallback (menuElement,
			      toggle ?
			      XmNvalueChangedCallback : XmNactivateCallback,
			      (XtCallbackProc) menu[j].callback,
			      menu[j].callbackData);

	       XmStringFree (label);
	       XmStringFree (acceleratorText);
	  }
     }


     /*
      * Link the label and separators into the menu.
      */
     label = XmStringCreateLtoR (menu[0].label, XmSTRING_DEFAULT_CHARSET);

     i = 0;
     XtSetArg (arglist[i], XmNlabelString, label); i++;
     XtSetArg (arglist[i], XmNmnemonic, menu[0].mnemonic); i++;
     XtSetArg (arglist[i], XmNsubMenuId, pane); i++;

     button = (Widget) XmCreateCascadeButton (parent,
					      concat (menu[0].id, "Menu"),
					      arglist, i);

     XtManageChild (button);

     XmStringFree (label);

     /*
      * Declare the help menu as the help menu for this application.
      */
     if (strcmp (menu[0].label, "Help") == 0) {
	  i = 0;
	  XtSetArg (arglist[i], XmNmenuHelpWidget, button); i++;
	  XtSetValues (parent, arglist, i);
     }

     return (pane);
}

     
Widget createSelectionMenu (parent, title, labels)
     Widget parent;
     String title;
     String labels;
{
     Widget pane;
     Widget menu;
     Widget menuElement;
     XmString string, accelerator;
     char s[512];

     String label;
     String labelSeparators = ","; /* <comma> */

     Arg arglist[5];
     Cardinal i;
     
     /*
      * Create a pulldown menu.
      */
     pane = (Widget) XmCreatePulldownMenu (parent,
					   concat (title, "Pane"), NULL, 0);

     /*
      * Add the first menu label.
      */
     if ((label = strtok (labels, labelSeparators)) != NULL) {
          string = XmStringCreateLtoR (label, XmSTRING_DEFAULT_CHARSET);

	  i = 0;
	  XtSetArg (arglist[i], XmNlabelString, string); i++;
	  menuElement = (Widget) XmCreatePushButtonGadget (pane,
							   concat (label, ""),
							   arglist, i);
	  XtManageChild (menuElement);
	  XtAddCallback (menuElement,
			 XmNactivateCallback,
			 (XtCallbackProc) menuParameterChange, label);

          XmStringFree (string);

	  /*
	   * Add the other labels.
	   */
	  while ((label = strtok (NULL, labelSeparators)) != NULL) {
               string = XmStringCreateLtoR (label, XmSTRING_DEFAULT_CHARSET);

	       i = 0;
	       XtSetArg (arglist[i], XmNlabelString, string); i++;
	       menuElement = (Widget) XmCreatePushButtonGadget (pane,
								concat (label,""),
								arglist, i);
	       XtManageChild (menuElement);
	       XtAddCallback (menuElement,
			      XmNactivateCallback,
			      (XtCallbackProc) menuParameterChange, label);

               XmStringFree (string);
	  }
     }

     /*
      * Link the label and separators into the menu.
      */
     string = XmStringCreateLtoR (title, XmSTRING_DEFAULT_CHARSET);

     i = 0;
     XtSetArg (arglist[i], XmNlabelString, string); i++;
     XtSetArg (arglist[i], XmNmnemonic, title[0]); i++;
     XtSetArg (arglist[i], XmNsubMenuId, pane); i++;
     menu = (Widget) XmCreateCascadeButton (parent,
					    concat (title, "Menu"),
					    arglist, i);
     XtManageChild (menu);

     XmStringFree (string);

     return (pane);
}


Widget createMenuArea (parent)
     Widget parent;
{
     Widget menuArea;

     /*
      * Create a menu bar.
      */
     menuArea = XmCreateMenuBar (parent, "menuArea", NULL, 0);
     XtManageChild (menuArea);

     /*
      * Add the application's main menus.
      */
     processPane = createMenu (menuArea, processMenu, XtNumber (processMenu));

     hostPane = createSelectionMenu (menuArea, "Host", resources.hosts);
     userPane = createSelectionMenu (menuArea, "User", resources.users);
     groupPane = createSelectionMenu (menuArea, "Group", resources.groups);
     terminalPane = createSelectionMenu (menuArea, "Terminal", resources.terminals);
     actionPane = createSelectionMenu (menuArea, "Action", resources.actions);

     viewPane = createMenu (menuArea, viewMenu, XtNumber (viewMenu));
     optionPane = createMenu (menuArea, optionMenu, XtNumber (optionMenu));
     helpPane = createMenu (menuArea, helpMenu, XtNumber (helpMenu));

     return (menuArea);
}


Widget createSelectionArea (parent)
     Widget parent;
{
     Widget selectionArea;
     Widget statusArea;
     Widget textArea;
     Widget infoArea;

     XmFontList fontList;
     Dimension width, height;
     
     Arg arglist[10];
     Cardinal i;
     
     /*
      * Create a form to contain the different fields.
      */
     selectionArea = (Widget) XmCreateForm (parent, "selectionArea", NULL, 0);
     XtManageChild (selectionArea);

     /*
      * Add an info zone.
      */
     i = 0;
     XtSetArg (arglist[i], XmNalignment, XmALIGNMENT_END); i++;
     XtSetArg (arglist[i], XmNwidth, 200); i++;
     XtSetArg (arglist[i], XmNtopAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNleftAttachment, XmATTACH_NONE); i++;
     XtSetArg (arglist[i], XmNrightAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNbottomAttachment, XmATTACH_FORM); i++;

     infoArea = (Widget) XmCreateLabel (selectionArea,
					"infoArea", arglist,i);
     XtManageChild (infoArea);

     /*
      * Add a status zone.
      */
     i = 0;
     XtSetArg (arglist[i], XmNalignment, XmALIGNMENT_BEGINNING); i++;
     XtSetArg (arglist[i], XmNtopAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNleftAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNrightAttachment, XmATTACH_WIDGET); i++;
     XtSetArg (arglist[i], XmNrightWidget, infoArea); i++;
     XtSetArg (arglist[i], XmNbottomAttachment, XmATTACH_FORM); i++;

     statusArea = (Widget) XmCreateLabel (selectionArea,
					  "statusArea", arglist,i);
     XtManageChild (statusArea);

     XtOverrideTranslations (statusArea,
			     XtParseTranslationTable (statusTranslations));

     i = 0;
     XtSetArg (arglist[i], XmNfontList, &fontList); i++;
     XtGetValues (statusArea, arglist, i);

     /*
      * Add an input zone.
      */
     i = 0;
     XtSetArg (arglist[i], XmNtopAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNleftAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNrightAttachment, XmATTACH_WIDGET); i++;
     XtSetArg (arglist[i], XmNrightWidget, infoArea); i++;
     XtSetArg (arglist[i], XmNbottomAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNfontList, fontList); i++;

     textArea = XmCreateText (selectionArea, "textArea", arglist, i);

     XtOverrideTranslations (textArea,
			     XtParseTranslationTable (textTranslations));

     i = 0;
     XtSetArg (arglist[i], XmNwidth, &width); i++;
     XtSetArg (arglist[i], XmNheight, &height); i++;
     XtGetValues (textArea, arglist, i);

     i = 0;
     XtSetArg (arglist[i], XmNwidth, width); i++;
     XtSetArg (arglist[i], XmNheight, height); i++;
     XtSetValues (statusArea, arglist, i);

     return (selectionArea);
}


Widget createProcessArea (parent)
     Widget parent;
{
     Arg arglist[10];
     Cardinal i;
     
     /*
      * Create a form to contain the processes windows.
      */
     processArea = (Widget) XmCreateForm (parent, "processArea", NULL, 0);
     XtManageChild (processArea);
     
     /*
      * Add a processes info header.
      */
     i = 0;
     XtSetArg (arglist[i], XmNalignment, XmALIGNMENT_BEGINNING); i++;
     XtSetArg (arglist[i], XmNtopAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNleftAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNrightAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNbottomAttachment, XmATTACH_NONE); i++;

     headerArea = (Widget) XmCreateLabelGadget (processArea,
						"headerArea", arglist, i);
     XtManageChild (headerArea);
     
     /*
      * Add a scrolled list.
      */
     i = 0;
     XtSetArg (arglist[i], XmNlistSizePolicy, XmCONSTANT); i++;
     XtSetArg (arglist[i], XmNselectionPolicy, XmEXTENDED_SELECT); i++;
     XtSetArg (arglist[i], XmNautomaticSelection, True); i++;

     XtSetArg (arglist[i], XmNtopAttachment, XmATTACH_WIDGET); i++;
     XtSetArg (arglist[i], XmNtopWidget, headerArea); i++;
     XtSetArg (arglist[i], XmNleftAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNrightAttachment, XmATTACH_FORM); i++;
     XtSetArg (arglist[i], XmNbottomAttachment, XmATTACH_FORM); i++;

     listArea = XmCreateScrolledList (processArea, "listArea", arglist, i);
     XtManageChild (listArea);

     return (processArea);
}


Widget createAllAreas (parent)
     Widget parent;
{
     Widget topWindow;
     Widget menuArea;
     Widget selectionArea;

     Arg arglist[2];
     Cardinal i;

     /*
      * Create the main window.
      */
     i = 0;
     XtSetArg (arglist[i], XmNshowSeparator, True); i++;
     topWindow = XmCreateMainWindow (parent, "topWindow", arglist, i);
     XtManageChild (topWindow);

     /*
      * Add a menu bar.
      */
     menuArea = createMenuArea (topWindow);

     /*
      * Add the processes area.
      */
     processArea = createProcessArea (topWindow);
     
     /*
      * Add the selection area.
      */
     selectionArea = createSelectionArea (topWindow);
     
     /*
      * Identify the main window's children.
      */
     XmMainWindowSetAreas (topWindow,
			   menuArea, selectionArea, NULL, NULL, processArea);

     return (topWindow);
}


int createNewView (widget, clientData, callData)
     Widget widget;
     caddr_t clientData;
     caddr_t callData;
{
}


void main (argc, argv)
     unsigned int argc;
     char *argv[];
{
     Arg arglist[3];
     Cardinal i;
     
     /*
      * Create the main window.
      */
     topLevel = XtInitialize ("topLevel", "XmPs",
			      xmPsOptions, XtNumber (xmPsOptions),
			      (Cardinal *) &argc, argv);

     i = 0;
     XtSetArg (arglist[i], XmNtitle, "Process Browser"); i++;
     XtSetValues (topLevel, arglist, i);
     
     /*
      * Add our specific actions.
      */
     XtAddActions (xmPsActions, XtNumber (xmPsActions));
     XtOverrideTranslations (topLevel,
			     XtParseTranslationTable (generalTranslations));

     /*
      * Get our resources.
      */
     XtGetApplicationResources (topLevel, &resources,
				xmPsResources, XtNumber (xmPsResources),
				NULL, 0);

     verifyOptions ();
     
     /*
      * Initialize some private data.
      */
     strcpy (currentHost, strtok (strdup (resources.hosts), ","));
     strcpy (currentUser, strtok (strdup (resources.users), ","));
     strcpy (currentGroup, strtok (strdup (resources.groups), ","));
     strcpy (currentTerminal, strtok (strdup (resources.terminals), ","));
     strcpy (currentAction, strtok (strdup (resources.actions), ","));

     /*
      * Create the busy cursor.
      */
     busyCursor = XCreateFontCursor (XtDisplay (topLevel), XC_watch);
     
     /*
      * Create all the widgets.
      */
     (void) createAllAreas (topLevel);

     /*
      * Display the processes information only if automatic refresh is
      * not in action.
      */
     if (! resources.automaticRefresh)
	  displayProcesses ();
     XtRealizeWidget (topLevel);

     if (resources.allProcesses ||
	 resources.noGroupLeader || resources.noNonTerminal) {
	  disableUser ();
	  disableGroup ();
	  disableTerminal ();
     }

     statusLineUpdate ();
     
     /*
      * Let the toolkit handle events from now on.
      */
     XtMainLoop ();
}

