/*
 * Copyright (C) 1991, 1992 Jean-Jacques Moreau.
 *
 * Permission to use, copy, modify and distribute this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  The author makes no representations
 * about the suitability of this software for any purpose.  It is
 * provided "as is" without express or implied warranty.
 *
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#ifdef GLOBAL
#define EXTERN
#else
#define EXTERN extern
#endif


EXTERN Widget topLevel;
EXTERN Widget processArea;
EXTERN Widget headerArea;
EXTERN Widget listArea;

EXTERN Widget searchDialog;

EXTERN Widget processPane;
EXTERN Widget hostPane;
EXTERN Widget userPane;
EXTERN Widget groupPane;
EXTERN Widget terminalPane;
EXTERN Widget actionPane;
EXTERN Widget viewPane;
EXTERN Widget optionPane;
EXTERN Widget helpPane;

#ifdef GLOBAL
char currentHost[512];
char currentUser[512];
char currentGroup[512];
char currentTerminal[512];
char currentAction[512];
#else
extern char currentHost[];
extern char currentUser[];
extern char currentGroup[];
extern char currentTerminal[];
extern char currentAction[];
#endif

Boolean internalBeautifier;

EXTERN Cursor busyCursor;


#define ACTION_SEARCH		101
#define ACTION_SEARCH_NEXT	102
#define ACTION_SEARCH_PREVIOUS	103
#define ACTION_NEW_VIEW		104

#define MENU_AUTOMATIC_REFRESH	201
#define MENU_FULL_LISTING	202
#define MENU_LONG_LISTING	203
#define MENU_CLUSTER_WIDE	204
#define MENU_ALL_PROCESSES	205
#define MENU_NO_GROUP_LEADER	206
#define MENU_NO_NON_TERMINAL	207
#define MENU_AUTOMATIC_MEMORY	208
#define MENU_BEAUTIFY		209
#define MENU_MULTIPLE_OCCURRENCES 210
#define MENU_PERIODIC_REFRESH	211
#define MENU_IGNORE_CASE	212
#define MENU_PRINT_SELECTED	213


extern void textParameterChange ();
extern int menuParameterChange ();
extern void reDisplayProcessesIfNecessary ();
extern int reDisplayProcesses ();
extern int selectAllProcesses ();
extern int unselectAllProcesses ();
extern int applyAction ();
extern int searchProcessPrompt ();
extern int searchProcess ();
extern int exitXmPs ();
extern int createNewView ();
extern int takeSnapshot ();
extern int changeOption ();
extern int versionXmPs ();
extern int printProcesses ();
extern String executeCommand ();
extern String pid ();
extern String ppid ();
extern void inputMode ();
extern void noInputMode ();
extern String concat ();
extern void parseHeader ();
extern void error ();


typedef struct _ResourceRec {
     String hosts;
     String actions;
     String users;
     String usersOption;
     String groups;
     String groupsOption;
     String terminals;
     String terminalsOption;

     String psCommand;
     String rshCommand;
     String printCommand;
     String beautifyCommand;

     int refreshPeriod;

     Boolean automaticRefresh;
     Boolean periodicRefresh;
     Boolean automaticMemory;
     Boolean beautify;
     Boolean multipleOccurrences;
     Boolean ignoreCase;
     Boolean printSelected;

     Boolean allProcesses;
     Boolean noGroupLeader;
     Boolean noNonTerminal;
     String allProcessesOption;
     String noGroupLeaderOption;
     String noNonTerminalOption;

     Boolean fullListing;
     Boolean longListing;
     String fullListingOption;
     String longListingOption;

     Boolean clusterWide;
     String clusterWideOption;
} ResourceStruct, *ResourceRec;

typedef struct _MenuDescRec {
     String label;		/* button label ("[-^]Label") */
     String id;			/* button internal identifier */
     char mnemonic;		/* button mnemonic */
     String acceleratorText;	/* button accelerator label */
     String accelerator;	/* button accelerator */
     int (*callback) ();	/* activateCallback or valueChangedCallback */
     XtPointer callbackData;	/* data passed to callback when called */
     Boolean *state;		/* initial state if toggle button ("^Label") */
} MenuDescStruct, *MenuDescRec;


EXTERN ResourceStruct resources;
